@@ -2,15 +2,20 @@
 
 # Memory addresses
 ADDRESS_R15 = 0x400f
+ADDRESS_ADC_POWER = 0x4062
+ADDRESS_ADC_BATTERY = 0x4063
 ADDRESS_LEVELA = 0x4064
 ADDRESS_LEVELB = 0x4065
+ADDRESS_PUSH_BUTTON = 0x4068
 ADDRESS_COMMAND_1 = 0x4070
+ADDRESS_COMMAND_2 = 0x4071
 ADDRESS_MA_MIN_VALUE = 0x4086
 ADDRESS_MA_MAX_VALUE = 0x4087
 ADDRESS_CURRENT_MODE = 0x407b
 ADDRESS_LCD_WRITE_PARAMETER_1 = 0x4180
 ADDRESS_LCD_WRITE_PARAMETER_2 = 0x4181
 ADDRESS_POWER_LEVEL = 0x41f4
+ADDRESS_BATTERY_LEVEL = 0x4203
 ADDRESS_LEVELMA = 0x420D
 ADDRESS_KEY = 0x4213
 
@@ -20,11 +25,9 @@ EEPROM_ADDRESS_FAVORITE_MODE = 0x800C
 
 # Commands
 COMMAND_START_FAVORITE_MODULE = 0x00
-COMMAND_SHOW_STATUS_SCREEN = 0x02
-COMMAND_SELECT_MENU_ITEM = 0x03
 COMMAND_EXIT_MENU = 0x04
+COMMAND_SHOW_MAIN_MENU = 0x0a
 COMMAND_NEW_MODE = 0x12
-COMMAND_WRITE_CHARACTER_TO_LCD = 0x13
 COMMAND_WRITE_STRING_TO_LCD = 0x15
 COMMAND_NO_COMMAND = 0xff
 
@@ -64,3 +67,9 @@ POWERLEVEL_HIGH = 0x03
 
 # Register 15 Bits
 REGISTER_15_ADCDISABLE = 0
+
+# Buttons
+BUTTON_MENU = 0x80
+BUTTON_OK = 0x20
+BUTTON_RIGHT = 0x40
+BUTTON_LEFT = 0x10
